/***********************************************************************************************************************
* @file  temp_sensors.c
*
* @brief temperature and ADC sensors source files
*
* (C) Copyright 2021 SmartAxiom,Inc All Rights Reserved
*
*  SMARTAXIOM CONFIDENTIAL
*  The source code contained or described herein and all documents related to
*  the source code ("Material") are owned by SmartAxiom,Inc or its
*  suppliers or licensor's. Title to the Material remains with SmartAxiom, Inc
*  or its suppliers and licensor's. The Material contain trade secrets and proprietary
*  and confidential information of SmartAxiom or its suppliers and licensor's.
*  The Material is protected by worldwide copyright and trade secret laws and
*  treaty provisions. No part of the Material may be used, copied, reproduced, modified,
*  published, uploaded, posted,transmitted, distributed, or disclosed in any way without
*  SmartAxiom prior express written permission.
*
*  No license under any patent, copyright, trade secret or other intellectual
*  property right is granted to or conferred upon you by disclosure or delivery
*  of the Materials, either expressly, by implication, inducement, estoppel or
*  otherwise. Any license under such intellectual property rights must be
*  express and approved by SmartAxiom in writing.
*
*/


#include <adc/temp_sensor.h>

/**
 * Initialize ADC controller for channel 0
 */

const adc_channel_cfg_t g_adc_channel_cfg =
{
    .scan_mask          = (uint32_t)ADC_CHANNEL_TEMPERATURE,
    .scan_mask_group_b  = 0,
    .priority_group_a   = (adc_group_a_t) 0,
    .add_mask           = 0,
    .sample_hold_mask   = 0,
    .sample_hold_states = 0,
};

/**
 * @brief Initialize ADC controller
 * @return FSP_SUCCESS or FSP ERROR CODE
 */

fsp_err_t temp_sensor_init(void)
{
	fsp_err_t err = FSP_SUCCESS;     // Error status

    /* Initializes the module. */
    err = R_ADC_Open(&g_adc0_ctrl, &g_adc0_cfg);

    if ( err != FSP_SUCCESS) {
    	return err;
    }

    /* Configures the ADC scan parameters */
    err = R_ADC_ScanCfg (&g_adc0_ctrl, &g_adc_channel_cfg);

    if ( err != FSP_SUCCESS) {
    	return err;
    }

    /* Start the ADC scan*/
    err = R_ADC_ScanStart (&g_adc0_ctrl);
    return err;
}

/**
 * @brief read ADC count and convert into temperature value
 * @param data
 * @return FSP_SUCCESS or FSP ERROR CODE
 */

fsp_err_t temp_sensor_get(uint16_t  *data) {
	fsp_err_t err = FSP_SUCCESS;     // Error statust
	uint16_t raw_adc;

	/* Read the result */
	err = R_ADC_Read(&g_adc0_ctrl, ADC_CHANNEL_TEMPERATURE, &raw_adc);

   *data = (uint16_t)(raw_adc - 328) / 16;

	return err;
}
/**
 * @brief Read chip die temperature through ADC
 * @param pADCTemp
 * @return SUCCESS or FAILED
 */

int read_ADC_Temp(uint8_t *pADCTemp)
{
    uint16_t Temp  = 0;
	sync_envelope_sensor_t adc_temp = { "Temperature",'F',0,NULL,0};

	/*Sensor data*/
	adc_temp.sensor_data_length = 64;

	/*Initialize sensor data and structure */
	adc_temp.sensor_data = pvPortMalloc(adc_temp.sensor_data_length);
	memset((char *)adc_temp.sensor_data,0,adc_temp.sensor_data_length);


	/*Read chip temperature*/
	int ret = temp_sensor_get(&Temp);
	sprintf((char *)adc_temp.sensor_data,"%d",Temp);

	if (ret == 0) {
		sprintf((char *)pADCTemp,"%s#%c#%c#%s#%c",(char *)adc_temp.sensor_name,adc_temp.sensor_type,
				                   adc_temp.sensor_data_length,adc_temp.sensor_data,adc_temp.sensor_event);

	}
	vPortFree(adc_temp.sensor_data);

	return ret;

}
