/***********************************************************************************************************************
* @file   beachhead-api.h
*
* @brief  RTC Hardware and driver interface functions
*
* (C) Copyright 2021 SmartAxiom,Inc All Rights Reserved
*
*  SMARTAXIOM CONFIDENTIAL
*  The source code contained or described herein and all documents related to
*  the source code ("Material") are owned by SmartAxiom,Inc or its
*  suppliers or licensor's. Title to the Material remains with SmartAxiom, Inc
*  or its suppliers and licensor's. The Material contain trade secrets and proprietary
*  and confidential information of SmartAxiom or its suppliers and licensor's.
*  The Material is protected by worldwide copyright and trade secret laws and
*  treaty provisions. No part of the Material may be used, copied, reproduced, modified,
*  published, uploaded, posted,transmitted, distributed, or disclosed in any way without
*  SmartAxiom prior express written permission.
*
*  No license under any patent, copyright, trade secret or other intellectual
*  property right is granted to or conferred upon you by disclosure or delivery
*  of the Materials, either expressly, by implication, inducement, estoppel or
*  otherwise. Any license under such intellectual property rights must be
*  express and approved by SmartAxiom in writing.
*
*******************************************************************************************************************/

#ifndef BEACHHEAD_API_BEACHHEAD_API_H_
#define BEACHHEAD_API_BEACHHEAD_API_H_

#include <beachhead.h>
#include <error.h>



/**
 * @brief
 * @param bh_context
 * @param payload
 * @param payload_len
 * @param outBuffer
 * @param oBufferLen
 * @return
 */
sync_envelope_err_type SessionDiscovery(void  *bh_context, uint8_t *payload,
		                                    uint32_t payload_len,uint8_t *outBuffer,uint32_t *oBufferLen);
/**
 *
 * @param bh_context
 * @param outBuffer
 * @param oBufferLen
 * @return
 */
sync_envelope_err_type CreateSession(void  *bh_context,uint8_t *outBuffer,uint32_t *oBufferLen);
/**
 *
 * @param bh_context
 * @param outBuffer
 * @param oBufferLen
 * @return
 */
sync_envelope_err_type VerifySession(void  *bh_context,uint8_t *outBuffer,uint32_t *oBufferLen);
/**
 *
 * @param bh_context
 * @param outBuffer
 * @param oBufferLen
 * @return
 */
sync_envelope_err_type AuthenticateSession(void  *bh_context,uint8_t *outBuffer,uint32_t *oBufferLen);
/**
 *
 * @param bh_context
 * @param outBuffer
 * @param oBufferLen
 * @return
 */
sync_envelope_err_type AddNewIdentity(void  *bh_context,uint8_t *outBuffer,uint32_t *oBufferLen);
/**
 *
 * @param bh_context
 * @param outBuffer
 * @param oBufferLen
 * @return
 */
sync_envelope_err_type ActivateIdentity(void *bh_context,uint8_t *outBuffer,uint32_t *oBufferLen);


/**
 *
 * @param bh_context
 * @param payload
 * @param payload_len
 * @param outBuffer
 * @param oBufferLen
 * @return
 */
sync_envelope_err_type CreateSyncBlock(void  *bh_context, uint8_t *payload,
                                     uint32_t payload_len,uint8_t *outBuffer,uint32_t *oBufferLen);

/**
 *
 * @param bh_context
 * @param outBuffer
 * @param oBufferLen
 * @return
 */
sync_envelope_err_type ResyncSyncBlock(void *bh_context,uint8_t *outBuffer,uint32_t *oBufferLen);

/**
 *
 * @param bh_context
 * @param en_blk
 * @return
 */
int RetrieveBuffer(const void *bh_context, sync_envelope_en_blk_t *en_blk);

/**
 * Release beachhead context
 * @param bh_context
 * @return SYNC_OK or SYNC_FAIL
 */
sync_envelope_err_type ReleaseContext(void *bh_context);

/**
 * @brief get beachhead state
 * @param bh_context
 * @return
 */
sync_envelope_msg_type GetState(const void *bh_context);

/**
 * @brief set beachhead state
 * @param bh_context
 * @param new_state
 * @return
 */
 sync_envelope_err_type SetState(const void *bh_context,sync_envelope_msg_type new_state);

 sync_envelope_err_type DeviceHeartBeat(void *bh_context, uint8_t *payload,
                                         uint32_t payload_len,uint8_t *outBuffer,uint32_t *oBufferLen);


#endif /* BEACHHEAD_API_BEACHHEAD_API_H_ */
