/***********************************************************************************************************************
 * @file   beachhead-app.c
 *
 * @brief  beachhead application sample which include
 *         - device provision,
 *         - sync
 *         - generate encrypt block
 *         - verifying a receiving block
 *         - decrypte  payload

 * (C) Copyright 2021 SmartAxiom,Inc All Rights Reserved
 *
 *  SMARTAXIOM CONFIDENTIAL
 *  The source code contained or described herein and all documents related to
 *  the source code ("Material") are owned by SmartAxiom,Inc or its
 *  suppliers or licensor's. Title to the Material remains with SmartAxiom, Inc
 *  or its suppliers and licensor's. The Material contain trade secrets and proprietary
 *  and confidential information of SmartAxiom or its suppliers and licensor's.
 *  The Material is protected by worldwide copyright and trade secret laws and
 *  treaty provisions. No part of the Material may be used, copied, reproduced, modified,
 *  published, uploaded, posted,transmitted, distributed, or disclosed in any way without
 *  SmartAxiom prior express written permission.
 *
 *  No license under any patent, copyright, trade secret or other intellectual
 *  property right is granted to or conferred upon you by disclosure or delivery
 *  of the Materials, either expressly, by implication, inducement, estoppel or
 *  otherwise. Any license under such intellectual property rights must be
 *  express and approved by SmartAxiom in writing.
 *
 */

#include "beachhead-provision/beachhead-provision.h"
#include "beachhead-app.h"

extern bool g_beachhead_common_initialized;
static uint32_t number_retrieve = 0;

int beachhead_common_init_service(void *bh_handler, bh_socket_t *bh_socket,
		uint8_t *state) {
	int ret = SYNC_ERR_FAIL;
	int bProvisionStatus = 0;
	const TickType_t xDelay = 1000 / portTICK_PERIOD_MS;

	if (bh_handler == NULL || bh_socket == NULL) {
		APP_ERR_PRINT("Invalid Handler, Error: %d...\r\n", ret);
		return ret;
	}
	while (!bProvisionStatus) {
		switch (*state) {
		case DEVICE_DISCOVERY:
			/*Discover responder*/
			ret = beachhead_discovery(bh_handler, bh_socket, state);
			if (ret != SYNC_BLK_DISCOVERY) {
				APP_PRINT("\r\nUnable to receive Discovery Block,Error: %d...",ret);
				vTaskDelay( xDelay );
			} else {
				APP_PRINT("\r\nDiscover responder, State: %d.....",*state);
			}
			break;
		case DEVICE_HANDSHAKE:
			/**
			 * Create session with responder
			 **/
			ret = beachhead_create_session(bh_handler, bh_socket, state);

			if (ret == SYNC_BLK_HANDSHAKE) {
				APP_PRINT("\r\nSetup a secure session, State: %d.....", *state);
			} else {
				APP_PRINT("\r\n Unable to setup secure session Error: %d...",ret);
			}
			break;

		case DEVICE_SESSION_VERIFY:
			/* *
			 * Verify responder and establish secure communication channel
			 * */
			ret = beachhead_verify_session(bh_handler, bh_socket, state);
			if (ret == SYNC_BLK_SESSSION_VERIFY) {
				APP_PRINT("\r\nVerify a secure session, State:%d...",*state);
			} else {
				APP_PRINT("\r\n Unable to verify secure session, Error: %d...",ret);
			}
			break;
		case DEVICE_REGISTER:
			/* *
			 * Register as new device with responder, if device token not validate or new device.
			 * The device will not communicate further until it received communication token from the responder.
			 * */
			APP_PRINT("\r\nStarting new device registration ...");
			ret = beachhead_register_device(bh_handler, bh_socket, state);
			if (ret == SYNC_BLK_REGISTERED) {
				APP_PRINT("\r\nDevice registration is completed, State: %d..." ,*state);
			} else {
				APP_PRINT("\r\nUnable to register new device Error: %d...",ret);
			}
			break;
		case DEVICE_AUTHENTICATE:
			/* *
			 * Device authenticate by the responder, It will verify device UUID and token,
			 * and communication token.
			 * */
			APP_PRINT("\r\n Starting Authentication mode ...");

			ret = beachhead_authenticate_session(bh_handler, bh_socket, state);

			if (ret == SYNC_BLK_AUTHENTICATE && *state == DEVICE_SEND_DATA) {
				APP_PRINT("\r\nDevice Authentication is completed, State: %d ...",*state);
				bProvisionStatus = true;
				g_beachhead_common_initialized = true;

			} else if (ret == SYNC_BLK_AUTHENTICATE && *state == DEVICE_SYNC) {
				APP_PRINT("\r\nDevice need to sync with the responder, State: %d...",*state);
				xQueueSend(g_publisher_queue, state, 0);
				bProvisionStatus = true;
			}
			else {
				APP_PRINT("\r\nUnable to authenticate device Error: %d...",ret);
		    }
			break;
#ifdef SELF_ACTIVATION
		case DEVICE_ACTIVATE:
			ret = beachhead_self_activate_device(bh_handler, bh_socket, state);
			if (ret == SYNC_BLK_ACTIVATE) {
			   APP_PRINT ("\r\nDevice activation completed and ready to communicate State: %d.....", *state);
			   /*device normal state and complete provision process */
				if (*state == DEVICE_SEND_DATA)
					bProvisionStatus = 1;

			 } else {
				 APP_PRINT("\r\nUnable to activate the device Error: %d....",ret);
			 }
			break;
#else
		case DEVICE_ACTIVATE:
			APP_PRINT("\r\nDevice is waiting for the activation...");
			ret = SYNC_BLK_REGISTERED;
			bProvisionStatus = 1;
			break;
#endif
		default:
			APP_ERR_PRINT("\r\nUnable to establish secure session with responders...");
			*state=DEVICE_DISCOVERY;
			bh_socket->bBroadcast = 0;
		}
	}
	return ret;
}

int beachhead_device_sync(void *bh_handler, bh_socket_t *bh_socket, uint8_t *beachhead_state)
{
	int ret = SYNC_ERR_FAIL;
	uint8_t pSyncBlk[SYNC_ENVELOPE_SZ] = { 0 };
	uint32_t oBufferlen = 0;

	if (bh_handler == NULL || bh_socket == NULL) {
		APP_PRINT("\r\n Invalid Handler, Error: %d... ",ret);
		return ret;
	}

	/*Build Device Sync Block*/
	ret = ResyncSyncBlock(bh_handler, pSyncBlk, &oBufferlen);

	if (ret == SYNC_OK) {
		/*Publish device sync block to responder*/
		ret = socket_send(bh_socket, pSyncBlk, (uint32_t) oBufferlen);

		if (ret == SYNC_OK) {
			APP_PRINT("\r\nDevice is syncing with the responder %d...", ret);
			xQueueSend(g_subscriber_queue, beachhead_state, 0);
		}

	} else {
		*beachhead_state = DEVICE_DISCOVERY;
		SetState(bh_handler, *beachhead_state);
	}

	return ret;
}

int beachhead_send_data(void *bh_handler, uint8_t *payload_t, uint16_t payload_len, uint8_t *pOutBuff, bh_socket_t *bh_socket,
		                uint8_t *beachhead_state)
{
	int ret = SYNC_ERR_FAIL;
	uint32_t oBufferlen = 0;
	uint8_t pSyncBlk[SYNC_ENVELOPE_SZ] = { RESET_VALUE };

	if (bh_handler == NULL || bh_socket == NULL) {
		APP_ERR_PRINT("\r\nInvalid Handler...");
		return ret;
	}

	if (pOutBuff == NULL || payload_t == NULL) {
		APP_ERR_PRINT("\r\nInvalid input or out buffer handler...");
		return ret;
	}

	/*Build Sync envelope Block*/
	ret = CreateSyncBlock(bh_handler, payload_t, payload_len, pSyncBlk, &oBufferlen);

	/*send encrypted data block to responder*/
	if (ret == SYNC_OK) {
		ret = socket_send(bh_socket, pSyncBlk, oBufferlen);
		if (ret == SYNC_OK) {
			xQueueSend(g_subscriber_queue, beachhead_state, 0);
		} else {
			APP_ERR_PRINT("\r\nUnable to send a block, Error %d", ret);
			*beachhead_state = DEVICE_DISCOVERY;

		}

	} else {
		APP_ERR_PRINT("\r\nUnable to Build sync block, Error %d", ret);
		*beachhead_state = DEVICE_DISCOVERY;
	}
	return ret;
}

int beachhead_received_data(void *bh_handler, bh_socket_t *bh_socket, uint8_t *beachhead_state, uint8_t *pOutBuff)
{
	int ret = SYNC_ERR_FAIL;
	uint8_t pSyncBlk[SYNC_ENVELOPE_SZ] = { RESET_VALUE };
	sync_envelope_en_blk_t en_blk;
	memset(pSyncBlk, 0, SYNC_ENVELOPE_SZ);

	ret = socket_receive(bh_socket, pSyncBlk, SYNC_ENVELOPE_SZ);

	if (ret > 0) {

		APP_PRINT("\r\nReceived total number of valid byte: %d", ret);

		/*Decrypted sync block*/
		memset((void*) &en_blk, 0, sizeof(sync_envelope_en_blk_t));
		en_blk.input_blk = pSyncBlk;
		en_blk.input_blk_len = (uint16_t) ret;
		en_blk.out_buffer = pOutBuff;
		en_blk.state = *beachhead_state;

		/*process received sync envelope to establish session keys*/
		ret = RetrieveBuffer(bh_handler, &en_blk);

		if (ret == SYNC_BLK_VERIFIED && en_blk.state == DEVICE_SEND_DATA) {
			/*Normal send data and reply to back to subscriber */
			APP_PRINT("\r\nDevice verified response state: %d....", en_blk.state);
			*beachhead_state = en_blk.state;

		} else if (ret == SYNC_BLK_NOT_VERIFIED && en_blk.state == DEVICE_SYNC) {

			/*Device resync with responder to verified previous communication*/
			APP_PRINT("\r\nDevice resync with host, state: %d....", en_blk.state);
			g_beachhead_common_initialized = false;
			*beachhead_state = en_blk.state;
			xQueueSend(g_publisher_queue, beachhead_state, 0);

		} else if (ret == SYNC_BLK_ACTIVATE  && en_blk.state == DEVICE_ACTIVATE) {
			/*Registered device  received activation token and transfer ownership */
			APP_PRINT("\r\nDevice activation starting state: %d....", en_blk.state);
			g_beachhead_common_initialized = false;
			*beachhead_state = en_blk.state;
			xQueueSend(g_publisher_queue, beachhead_state, 0);

		}else if (ret == SYNC_BLK_STATUS && en_blk.state == DEVICE_STATUS)  {
			/*Device return his ownership and status*/
			APP_PRINT("\r\nStatus %s",pOutBuff);
			xQueueSend(g_publisher_queue, beachhead_state, 0);

		} else if (ret == SYNC_BLK_RESYNC && en_blk.state == DEVICE_SEND_DATA) {
			APP_PRINT("\r\nDevice re-sync completed, state: %d",en_blk.state);
			g_beachhead_common_initialized = true;
			*beachhead_state = en_blk.state;

		} else if (ret == SYNC_BLK_ACTIVATE && en_blk.state == DEVICE_SEND_DATA) {
			APP_PRINT("\r\nDevice activation has completed and starting normal operation, state: %d",en_blk.state);
			g_beachhead_common_initialized = true;
			xTimerStart(g_new_timer0, 0);
			*beachhead_state  = en_blk.state;

		} else {
			if (ret == SYNC_ERR_SIGNED) {
				APP_PRINT("\r\n Unable to established  secure session...");
			} else if (ret == SYNC_ERR_DEVICE_UUID) {
				APP_PRINT("\r\nInvalide UUID");
			} else {
				APP_ERR_PRINT("\r\nError in retrieve buffer...");
			}

			*beachhead_state = DEVICE_DISCOVERY;
			SetState(bh_handler, *beachhead_state);
			xQueueSend(g_publisher_queue, beachhead_state, 0);
		}
	} else {
		/*Error on received function and try to retrieve block NUM_RETRIEVE */
		if (*beachhead_state != DEVICE_HEARTBEAT) {
			if ((number_retrieve <= NUM_RETRIEVE) && ret < 0) {
				number_retrieve++;
			} else {
				number_retrieve = 0;
				*beachhead_state = DEVICE_DISCOVERY;
				SetState(bh_handler, *beachhead_state);
				xQueueSend(g_publisher_queue, beachhead_state, 0);
			}
		}
	}

	return ret > 0 ? SYNC_OK : SYNC_ERR_FAIL;
}

int beachhead_heart_beat(void *bh_handler, bh_socket_t *bh_socket, uint8_t *beachhead_state)
{
	int ret = SYNC_ERR_FAIL;
	uint32_t oBufferlen = 0;
	uint8_t payload_t[] = { HEART_BEAT };
	uint8_t pSyncBlk[SYNC_ENVELOPE_SZ] = { RESET_VALUE };

	if (bh_handler == NULL || bh_socket == NULL) {
		APP_ERR_PRINT("\r\nInvalid Handler...");
		return ret;
	}

	/*Build Sync envelope Block*/
	ret = DeviceHeartBeat(bh_handler, payload_t,
			(uint32_t) strlen((char*) HEART_BEAT), pSyncBlk, &oBufferlen);

	/*send encrypted data block to responder*/
	if (ret == SYNC_OK) {
	    ret = socket_send(bh_socket, pSyncBlk, oBufferlen);

		if (ret == SYNC_OK) {
			xQueueSend(g_subscriber_queue, beachhead_state, 0);
		}

	} else {
		APP_ERR_PRINT("\r\n Unable to build heart beat block %d", ret);
	}
	return ret;
}

void beachhead_close(bh_socket_t *bh_socket)
{

	/*Reset Subscriber event queue*/
	xQueueReset(g_subscriber_queue);

	/*Reset publisher event queue*/
	xQueueReset(g_publisher_queue);

	/*close Socket*/
	if (bh_socket->xSocket != FREERTOS_INVALID_SOCKET) {
		socket_close(bh_socket);
	}
}


int beachhead_activate_device(void *bh_handler, bh_socket_t *bh_socket,uint8_t *beachhead_state)
{
	int ret = SYNC_ERR_FAIL;
    uint8_t recv_blk[SYNC_ENVELOPE_SZ] = {0};
    uint32_t oBufferlen = 0 ;

    /*Request to responder to establish session keys*/
	 ret = ActivateIdentity(bh_handler,recv_blk,&oBufferlen);

	 if (ret == SYNC_OK) {
	  	APP_PRINT("\r\nStarting activation process...");

	  	/*Send activation block to local BlockLock or send to proxy at cloud */
	  	ret = socket_send(bh_socket,recv_blk,oBufferlen);

	  	if (ret == SYNC_OK) {
	  	   APP_PRINT("\r\nSuccessfully send device activation request %d...", ret)
		   xQueueSend(g_subscriber_queue, beachhead_state, 0);
	  	}

	 } else {
	    APP_PRINT("\r\nUnable to activate the device....\r\n");
	 }

	 return ret;
}
