/***********************************************************************************************************************
* @file   beachhead-provision.c
*
* @brief  beachhead device provision source file
*
* (C) Copyright 2021 SmartAxiom,Inc All Rights Reserved
*
*  SMARTAXIOM CONFIDENTIAL
*  The source code contained or described herein and all documents related to
*  the source code ("Material") are owned by SmartAxiom,Inc or its
*  suppliers or licensor's. Title to the Material remains with SmartAxiom, Inc
*  or its suppliers and licensor's. The Material contain trade secrets and proprietary
*  and confidential information of SmartAxiom or its suppliers and licensor's.
*  The Material is protected by worldwide copyright and trade secret laws and
*  treaty provisions. No part of the Material may be used, copied, reproduced, modified,
*  published, uploaded, posted,transmitted, distributed, or disclosed in any way without
*  SmartAxiom prior express written permission.
*
*  No license under any patent, copyright, trade secret or other intellectual
*  property right is granted to or conferred upon you by disclosure or delivery
*  of the Materials, either expressly, by implication, inducement, estoppel or
*  otherwise. Any license under such intellectual property rights must be
*  express and approved by SmartAxiom in writing.
*
*/
#include "beachhead-provision.h"

#define BH_WELCOME "WELCOME BEACHEAD"

int beachhead_discovery(void *bh_handler, bh_socket_t *bh_socket,uint8_t *beachhead_state)
{
	int ret = SYNC_ERR_FAIL;
	sync_envelope_en_blk_t en_blk;
	uint8_t recv_blk[SYNC_ENVELOPE_SZ] = {0};
	uint8_t process_blk[SYNC_ENVELOPE_PAYLOAD_SZ] = {0};
	uint8_t remote_addr[16];
	uint32_t oBufferlen = 0 ;

	if (bh_socket->bBroadcast == 0 ) {
	   bh_socket->vPort = PORT_B;
	   bh_socket->bBroadcast = 1;
	   bh_socket->socket_type = FREERTOS_SOCK_DGRAM;
	   bh_socket->socket_protocol = FREERTOS_IPPROTO_UDP;
	   socket_init(bh_socket);
    }

   /*Build Discovery Block*/
   ret = SessionDiscovery(bh_handler,(uint8_t *)PROVISION_MSG,
						  (uint16_t)strlen((char *)PROVISION_MSG),
							  recv_blk,&oBufferlen);
   if (ret == SYNC_OK ) {
	  /*Broadcast to local BlockLock or send message to proxy at cloud */
	  ret = socket_send(bh_socket,recv_blk,oBufferlen);

	  if (ret == SYNC_OK ) {
		  APP_PRINT("\r\nEnabling broadcast messages to discover the new session...");

		  /*respond from the BlockLock */
	     ret = socket_receive(bh_socket,recv_blk,SYNC_ENVELOPE_SZ);

	     if (ret > 0 ) {

	    	APP_PRINT("\r\nNetwork Discovered...");
			en_blk.input_blk = &recv_blk[0];
			en_blk.input_blk_len = (uint16_t)ret;
			en_blk.out_buffer = process_blk;
			en_blk.state = *beachhead_state;

		    /*process received sync envelope for discovery*/
		    ret = RetrieveBuffer(bh_handler, &en_blk);
		    *beachhead_state = en_blk.state;

		    APP_PRINT("/r/nReceived Buffer %s len %d",en_blk.out_buffer,en_blk.out_buffer_len);

		    if (ret == SYNC_BLK_DISCOVERY) {
			   if (strncmp((char *)en_blk.out_buffer,BH_WELCOME,strlen((char *)BH_WELCOME)) == 0) {
				  /* *
				   * after verified responder message and closed broadcast socket connection,
				   * Open new unicast connection with remote responder IP address at port 18088
				   */
				  APP_PRINT("\r\n Discovery complete...");
				  GetRemoteAddress(bh_socket,remote_addr);
				  beachhead_close(bh_socket);

				  APP_PRINT("\r\nRemote Address %s Port %d ...", remote_addr,PORT_B);
				  bh_socket->vPort = PORT_B;
				  bh_socket->bBroadcast = 0;
#ifdef FREERTOS_UDP
			      bh_socket->socket_type = FREERTOS_SOCK_DGRAM;
				  bh_socket->socket_protocol = FREERTOS_IPPROTO_UDP;
#else
				  bh_socket->socket_type = FREERTOS_SOCK_STREAM;
				  bh_socket->socket_protocol = FREERTOS_IPPROTO_TCP;
#endif
				  memset(bh_socket->address,0,strlen((char *)bh_socket->address));
				  strncpy((char *)bh_socket->address,(char *)remote_addr,strlen((char *)remote_addr));
				  socket_init(bh_socket);

			   }
		    } else {
		    	 APP_PRINT("\r\nError in unpacking the discovery block %d",ret);
		    }
	     } else {
	    	 APP_PRINT("\r\nError in receiving from responder %d",ret);
	     }
      } else {
    	  APP_PRINT("\r\nError in connecting to responder %d",ret);
      }
   }
   return ret;
}


int beachhead_create_session(void *bh_handler, bh_socket_t *bh_socket,uint8_t *beachhead_state)
{
   int ret = SYNC_ERR_FAIL;
   sync_envelope_en_blk_t en_blk;
   uint8_t recv_blk[SYNC_ENVELOPE_SZ] = {0};
   uint8_t process_blk[SYNC_ENVELOPE_PAYLOAD_SZ] = {0};
   uint32_t oBufferlen = 0 ;

   /*Request to responder for a create secure session*/
   ret = CreateSession(bh_handler,recv_blk,&oBufferlen);
   if (ret == SYNC_OK) {

	   /*Broadcast to local BlockLock or send message to proxy in cloud */
	   ret = socket_send(bh_socket,recv_blk,oBufferlen);
	   if ( ret == SYNC_OK) {
		   /*Received responder response*/
		   ret =  socket_receive(bh_socket,recv_blk,(uint16_t)SYNC_ENVELOPE_SZ);

		   if ( ret > 0 ) {
			   APP_PRINT("\r\nComplete Handshake process...");
			   en_blk.input_blk = recv_blk;
			   en_blk.input_blk_len = (uint16_t)ret;
			   en_blk.out_buffer = process_blk;
			   en_blk.state = *beachhead_state;

			   /*process received to established secure connection with responder*/
			   ret = RetrieveBuffer(bh_handler, &en_blk);
			   /*set return state*/
			   *beachhead_state = en_blk.state;
		   }
		}
    } else {
	  /* Unable to established a secure session with responder
	   * reset to state to the discovery mode.So device will be established session.
	   * */
    	*beachhead_state = DEVICE_DISCOVERY;
   }

   return ret;
}


int beachhead_verify_session(void *bh_handler, bh_socket_t *bh_socket,uint8_t *beachhead_state)
{
   int ret = SYNC_ERR_FAIL;
   sync_envelope_en_blk_t en_blk;
   uint8_t recv_blk[SYNC_ENVELOPE_SZ] = {0};
   uint8_t process_blk[SYNC_ENVELOPE_PAYLOAD_SZ] = {0};
   uint32_t oBufferlen = 0 ;

   /*Request to responder to verify secure session*/
   ret = VerifySession(bh_handler,recv_blk,&oBufferlen);

   if ( ret == SYNC_OK ) {
	   /*Broadcast to local BlockLock or send message to proxy in cloud */
	   ret = socket_send(bh_socket,recv_blk,oBufferlen);
	   if ( ret == SYNC_OK) {
		   /*Received responder response*/
		   ret =  socket_receive(bh_socket,recv_blk,(uint16_t)SYNC_ENVELOPE_SZ);
		   if ( ret > 0) {
			   APP_PRINT("\r\nVerify secure session connection...");
			   en_blk.input_blk = recv_blk;
			   en_blk.input_blk_len = (uint16_t)ret;
			   en_blk.out_buffer = process_blk;
			   en_blk.state = *beachhead_state;

			   /*process received sync envelope to establish session keys*/
			   ret = RetrieveBuffer(bh_handler, &en_blk);
			   *beachhead_state = en_blk.state;
		   }
	   }
	} else {
	   *beachhead_state = DEVICE_DISCOVERY;
   }

   return ret;
}

int beachhead_authenticate_session(void *bh_handler, bh_socket_t *bh_socket,uint8_t *beachhead_state)
{
	int ret = SYNC_ERR_FAIL;
	sync_envelope_en_blk_t en_blk;
	uint8_t recv_blk[SYNC_ENVELOPE_SZ] = {0};
	uint8_t process_blk[SYNC_ENVELOPE_PAYLOAD_SZ] = {0};
	uint32_t oBufferlen = 0 ;

	 APP_PRINT("\r\nDevice already provisioned and need to re-authenticate...");

    /*Request to responder to establish session keys*/
    ret = AuthenticateSession(bh_handler,recv_blk, &oBufferlen);

    if (ret == SYNC_OK ) {
	   APP_PRINT("\r\nStarting device authentication process...");

	   /*Send authentication request to local BlockLock or send to proxy at cloud */
	   ret = socket_send(bh_socket,recv_blk,oBufferlen);

	   if (ret == SYNC_OK) {
		  /*Received response from the BlockLock for activate a new device*/
		  ret =  socket_receive(bh_socket,recv_blk,(uint16_t)SYNC_ENVELOPE_SZ);

		  if (ret > 0) {
		     memset((void *)&en_blk,0,sizeof(sync_envelope_en_blk_t));
		     en_blk.input_blk = recv_blk;
		     en_blk.input_blk_len = (uint16_t)ret;
		     en_blk.out_buffer = process_blk;
		     en_blk.state = *beachhead_state;

		     /*process authentication block*/
		     ret = RetrieveBuffer( bh_handler, &en_blk);
		     *beachhead_state = en_blk.state;

		  } else {
			*beachhead_state = DEVICE_DISCOVERY;

			if (GetState(bh_handler) != DEVICE_DISCOVERY)
			    SetState(bh_handler,*beachhead_state);
		 }
	   } else {
		  *beachhead_state = DEVICE_DISCOVERY;
		  if (GetState(bh_handler) != DEVICE_DISCOVERY)
		     SetState(bh_handler,*beachhead_state);
	  }
    } else {
	   APP_PRINT("\r\nUnable to build authenticate request...");
	   *beachhead_state = DEVICE_DISCOVERY;
	   SetState(bh_handler,*beachhead_state);
    }

	return ret;
}

int beachhead_register_device(void *bh_handler, bh_socket_t *bh_socket,uint8_t *beachhead_state)
{
	int ret = SYNC_ERR_FAIL;
	sync_envelope_en_blk_t en_blk;
	uint8_t recv_blk[SYNC_ENVELOPE_SZ] = {0};
	uint8_t process_blk[SYNC_ENVELOPE_PAYLOAD_SZ] = {0};
	uint32_t oBufferlen = 0 ;

    /*Request to responder to register new device*/
    ret = AddNewIdentity(bh_handler, recv_blk,&oBufferlen);

    if (ret == SYNC_OK) {
	   APP_PRINT("\r\nStarting device registration process...");

	   /*Broadcast to local BlockLock or send message to proxy at cloud */
	   ret = socket_send(bh_socket,recv_blk,oBufferlen);
	   if ( ret == SYNC_OK) {
		   /*Received confirmation from responder for register new device*/
		   ret =  socket_receive(bh_socket,recv_blk,(uint16_t)SYNC_ENVELOPE_SZ);

		   if ( ret > 0 ) {
			  memset((void *)&en_blk,0,sizeof(sync_envelope_en_blk_t));
			  en_blk.input_blk = recv_blk;
			  en_blk.input_blk_len = (uint16_t)ret;
			  en_blk.out_buffer = process_blk;
			  en_blk.state = *beachhead_state;

			  /*process received to confirmed that new device add to BlockLock*/
			  ret = RetrieveBuffer(bh_handler, &en_blk);
			  *beachhead_state = en_blk.state;
		   }
	   }

    } else {
	  APP_PRINT("\r\nUnable to register the device...");
	  *beachhead_state = DEVICE_SESSION_VERIFY;
	  SetState(bh_handler,*beachhead_state);
    }

    return ret;
}

#ifdef SELF_ACTIVATION
int beachhead_self_activate_device(void *bh_handler, bh_socket_t *bh_socket,uint8_t *beachhead_state)
{
	int ret = SYNC_ERR_FAIL;
    sync_envelope_en_blk_t en_blk;
    uint8_t recv_blk[SYNC_ENVELOPE_SZ] = {0};
    uint8_t process_blk[SYNC_ENVELOPE_PAYLOAD_SZ] = {0};
    uint32_t oBufferlen = 0 ;

    /*Request to responder to establish session keys*/
	 ret = ActivateIdentity(bh_handler,recv_blk,&oBufferlen);

	 if (ret == SYNC_OK) {
	  	wmprintf("\r\nStarting device Activation process...");

	  	/*Send activation block to local BlockLock or send to proxy at cloud */
	  	ret = socket_send(bh_socket,recv_blk,oBufferlen);
	  	if (ret == SYNC_OK) {
	  	   /*Received confirmation from the BlockLock for activate a new device*/
	  	   ret =  socket_receive(bh_socket,recv_blk,(uint16_t)SYNC_ENVELOPE_SZ);
	  	   if (ret > 0) {
	  		   memset((void *)&en_blk,0,sizeof(sync_envelope_en_blk_t));
	  		   en_blk.input_blk = recv_blk;
	  		   en_blk.input_blk_len = (uint16_t)ret;
	  		   en_blk.out_buffer = process_blk;
	  		   en_blk.state = *beachhead_state;

	  		   /*process received sync envelope to establish session keys*/
	  		   ret = RetrieveBuffer(bh_handler, &en_blk);
	  		   *beachhead_state = en_blk.state;

	  	   } else {
	  		 *beachhead_state = DEVICE_DISCOVERY;
			   if (GetState(bh_handler) != DEVICE_DISCOVERY)
				  SetState(bh_handler,*beachhead_state);
	  	   }
	  	 } else {
	  		*beachhead_state = DEVICE_DISCOVERY;
	        if (GetState(bh_handler) != DEVICE_DISCOVERY)
			   SetState(bh_handler,*beachhead_state);
	    }
	 } else {
	    wmprintf("\r\nUnable to Activate the device...");
	    *beachhead_state = DEVICE_DISCOVERY;
	 }

	 return ret;
}
#endif





