/***********************************************************************************************************************
* @file client.h
*
* @brief  Networking and Socket
*
* (C) Copyright 2021 SmartAxiom,Inc All Rights Reserved
*
*  SMARTAXIOM CONFIDENTIAL
*  The source code contained or described herein and all documents related to
*  the source code ("Material") are owned by SmartAxiom,Inc or its
*  suppliers or licensor's. Title to the Material remains with SmartAxiom, Inc
*  or its suppliers and licensor's. The Material contain trade secrets and proprietary
*  and confidential information of SmartAxiom or its suppliers and licensor's.
*  The Material is protected by worldwide copyright and trade secret laws and
*  treaty provisions. No part of the Material may be used, copied, reproduced, modified,
*  published, uploaded, posted,transmitted, distributed, or disclosed in any way without
*  SmartAxiom prior express written permission.
*
*  No license under any patent, copyright, trade secret or other intellectual
*  property right is granted to or conferred upon you by disclosure or delivery
*  of the Materials, either expressly, by implication, inducement, estoppel or
*  otherwise. Any license under such intellectual property rights must be
*  express and approved by SmartAxiom in writing.
*
*/

#ifndef INCLUDE_CLIENT_H_
#define INCLUDE_CLIENT_H_

#include "beachhead_config.h"
#include <inttypes.h>

#define PORT_B               18088
#define READ_TIMEOUT         200
#define PACKET_SIZE          256
#define NUMBER_OF_SESSIONS   25
#define NUM_RETRIEVE         5

/**
 * socket structure
 */
typedef struct bh_socket {
#ifdef FREERTOS
	Socket_t xSocket;
	struct freertos_sockaddr xRemoteAddress;
#else
	struct sockaddr_in xSocket;
#endif
	uint8_t  bBroadcast;
	uint8_t  socket_type;
	uint8_t  socket_protocol;
	uint16_t vPort;
	uint8_t  address[16];
} bh_socket_t;

/**
 * Initialize network stack and acquired IP address
 * @return
 */
BaseType_t IP_Setup(uint8_t *ether_mac_addr);

/**
 *
 */
void ip_init(void);
/**
 *
 */
void socket_close(bh_socket_t *pSocket);

/**
 *
 * @param vPort
 * @param pAddress
 * @return
 */
int socket_open(bh_socket_t *pSocket);

/**
 *
 * @return
 */
int socket_connect(bh_socket_t *pSocket);

/**
 *
 * @param buff
 * @param buf_len
 * @return
 */
int socket_send(bh_socket_t *pSocket,uint8_t *buff,uint32_t buf_len);

/**
 *
 * @param recv_buff
 * @param recv_buff_len
 * @return
 */
int socket_receive(bh_socket_t *pSocket,uint8_t *recv_buff,uint16_t recv_buff_len);

/**
 *
 * @param bh_socket
 * @return
 */
int socket_init(bh_socket_t *bh_socket);

/**
 *
 * @param pSocket
 * @param pAddress
 */

void GetRemoteAddress(bh_socket_t *pSocket,uint8_t *cIPAddressString);



#endif /* INCLUDE_CLIENT_H_ */
