/***********************************************************************************************************************
* @file rtc.h
*
* @brief  RTC Hardware and driver interface functions
*
* (C) Copyright 2021 SmartAxiom,Inc All Rights Reserved
*
*  SMARTAXIOM CONFIDENTIAL
*  The source code contained or described herein and all documents related to
*  the source code ("Material") are owned by SmartAxiom,Inc or its
*  suppliers or licensor's. Title to the Material remains with SmartAxiom, Inc
*  or its suppliers and licensor's. The Material contain trade secrets and proprietary
*  and confidential information of SmartAxiom or its suppliers and licensor's.
*  The Material is protected by worldwide copyright and trade secret laws and
*  treaty provisions. No part of the Material may be used, copied, reproduced, modified,
*  published, uploaded, posted,transmitted, distributed, or disclosed in any way without
*  SmartAxiom prior express written permission.
*
*  No license under any patent, copyright, trade secret or other intellectual
*  property right is granted to or conferred upon you by disclosure or delivery
*  of the Materials, either expressly, by implication, inducement, estoppel or
*  otherwise. Any license under such intellectual property rights must be
*  express and approved by SmartAxiom in writing.
*
*/

#ifndef RTC_RTC_H_
#define RTC_RTC_H_

#include <time.h>
#include <r_rtc.h>
#include <r_rtc_api.h>

#define SET_TIMEOUT(t, ms)								\
	do {												\
		(t)->tv_sec = (ms)/1000;						\
		(t)->tv_usec = ((ms)%1000) * 1000;				\
	} while (0)


/**
 * @brief  Initialize RTC controller
 * @return FSP_SUCCESS or FSP_ERR_NOT_OPEN
 */
int rtc_user_init();

/**
 * @brief       Get current time
 * @return      time_t
 */
time_t get_user_time(void);
/**
 * @brief set RTC controller with current time
 * @param time : current time in time_t value
 */
void set_user_time(time_t time);

#endif /* RTC_RTC_H_ */
