/***********************************************************************************************************************
* @file   common_utils.h
*
* @brief  Contains macros, data structures and functions used  common to the beachhead
*         sample code
*
* (C) Copyright 2021 SmartAxiom,Inc All Rights Reserved
*
*  SMARTAXIOM CONFIDENTIAL
*  The source code contained or described herein and all documents related to
*  the source code ("Material") are owned by SmartAxiom,Inc or its
*  suppliers or licensor's. Title to the Material remains with SmartAxiom, Inc
*  or its suppliers and licensor's. The Material contain trade secrets and proprietary
*  and confidential information of SmartAxiom or its suppliers and licensor's.
*  The Material is protected by worldwide copyright and trade secret laws and
*  treaty provisions. No part of the Material may be used, copied, reproduced, modified,
*  published, uploaded, posted,transmitted, distributed, or disclosed in any way without
*  SmartAxiom prior express written permission.
*
*  No license under any patent, copyright, trade secret or other intellectual
*  property right is granted to or conferred upon you by disclosure or delivery
*  of the Materials, either expressly, by implication, inducement, estoppel or
*  otherwise. Any license under such intellectual property rights must be
*  express and approved by SmartAxiom in writing.
*
*******************************************************************************************************************/

#ifndef COMMON_UTILS_H_
#define COMMON_UTILS_H_

/* generic headers */
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "hal_data.h"

/* SEGGER RTT and error related headers */
#include "SEGGER_RTT/SEGGER_RTT.h"


#define BIT_SHIFT_8  (8u)
#define SIZE_64      (64u)

#define LVL_ERR      (1u)       /* error conditions   */

#define RESET_VALUE             (0x00)

#if defined (BOARD_RA6M3_EK) || defined (BOARD_RA6M3G_EK)
#define KIT_NAME                "EK-RA6M3-Beachhead"
#else
#define KIT_NAME                "EK-RA6M4-Beachhead"
#endif

#define BH_VERSION              ("1.0")
#define MODULE_NAME             "Beachhead-SDK using EK-RA6M3"
#define BANNER_INFO             "\r\n********************************************************************************"\
                                "\r\n*   SmartAxiom Beachhead Decentralized Security Example                        *"\
                                "\r\n*   Project for Renesas EK-RA6M3 board                                         *"\
                                "\r\n*   Example Project Version %s                                                 *"\
                                "\r\n*   Flex Software Pack Version  %d.%d.%d                                       *"\
                                "\r\n********************************************************************************"\
                                "\r\nRefer to readme.txt file for more details on Example Project and               *"\
                                "\r\nBeachhead User's Manual for more information about Beachhead-SDK               *"\
                                "\r\n********************************************************************************\r\n"

#define ETH_PREINIT             "\r\n \r\n--------------------------------------------------------------------------------"\
                                "\r\nEthernet adapter Configuration for Renesas "KIT_NAME": Pre IP Init       "\
                                "\r\n--------------------------------------------------------------------------------\r\n\r\n"

#define ETH_POSTINIT             "\r\n \r\n--------------------------------------------------------------------------------"\
                                "\r\nEthernet adapter Configuration for Renesas "KIT_NAME": Post IP Init       "\
                                "\r\n--------------------------------------------------------------------------------\r\n\r\n"

#define SEGGER_INDEX            (0)

#define APP_PRINT(fn_, ...)      SEGGER_RTT_printf (SEGGER_INDEX,(fn_), ##__VA_ARGS__);

#define APP_ERR_PRINT(fn_, ...)  if(LVL_ERR)\
        SEGGER_RTT_printf (SEGGER_INDEX, "[ERR] In Function: %s(), %s",__FUNCTION__,(fn_),##__VA_ARGS__);

#define APP_ERR_TRAP(err)        if(err) {\
        SEGGER_RTT_printf(SEGGER_INDEX, "\r\nReturned Error Code: 0x%x  \r\n", err);\
        __asm("BKPT #0\n");} /* trap upon the error  */

#define APP_READ(read_data)     SEGGER_RTT_Read (SEGGER_INDEX, read_data, sizeof(read_data));

#define APP_CHECK_DATA          SEGGER_RTT_HasKey()

void AsciitoHex(unsigned char *ascii, unsigned char *frame,  int len);

void StringtoHex(uint8_t *string, uint8_t *output);
#endif /* COMMON_UTILS_H_ */
