/***********************************************************************************************************************
* @file  leds.c
*
* @brief Controls various leds
*
* (C) Copyright 2021 SmartAxiom,Inc All Rights Reserved
*
*  SMARTAXIOM CONFIDENTIAL
*  The source code contained or described herein and all documents related to
*  the source code ("Material") are owned by SmartAxiom,Inc or its
*  suppliers or licensor's. Title to the Material remains with SmartAxiom, Inc
*  or its suppliers and licensor's. The Material contain trade secrets and proprietary
*  and confidential information of SmartAxiom or its suppliers and licensor's.
*  The Material is protected by worldwide copyright and trade secret laws and
*  treaty provisions. No part of the Material may be used, copied, reproduced, modified,
*  published, uploaded, posted,transmitted, distributed, or disclosed in any way without
*  SmartAxiom prior express written permission.
*
*  No license under any patent, copyright, trade secret or other intellectual
*  property right is granted to or conferred upon you by disclosure or delivery
*  of the Materials, either expressly, by implication, inducement, estoppel or
*  otherwise. Any license under such intellectual property rights must be
*  express and approved by SmartAxiom in writing.
*
*/

#include "beachhead_config.h"
#include "leds.h"


static led_ctrl_t cur_led_state[3] = {LED_OFF};
static bsp_led_t last_sw_toggle = BSP_LED_LED1;


void turn_led_on(bsp_led_t ltype)
{
	R_IOPORT_PinWrite(&g_ioport_ctrl, g_bsp_leds.p_leds[ltype], LED_ON);
	cur_led_state[ltype] = LED_ON;
}

void turn_led_off(bsp_led_t ltype)
{
	R_IOPORT_PinWrite(&g_ioport_ctrl, g_bsp_leds.p_leds[ltype], LED_OFF);
	cur_led_state[ltype] = LED_OFF;
}

void led_toggle(bsp_led_t ltype)
{
	if ( LED_OFF == cur_led_state[ltype] ) {
		turn_led_on(ltype);
	}
	else {
		turn_led_off(ltype);
	}
	/*Assign*/
}

void led_sw_status(uint8_t *pSWEvent,bsp_led_t ltype)
{
	sync_envelope_sensor_t sw_event =  {"SmartSwitch",'I',strlen(SW_LED_OFF),NULL,LED_OFF};

	if ( ltype == BSP_LED_LED1 ) {
		sw_event.sensor_event = cur_led_state[ltype] == LED_ON ? LED_OFF : LED_ON ;
	} else if (ltype == BSP_LED_LED2)  {
		sw_event.sensor_event = cur_led_state[ltype] == LED_ON ? LED_OFF : LED_ON;
	}

	last_sw_toggle = ltype;

	/*allocate memory*/
	sw_event.sensor_data =  pvPortMalloc(sw_event.sensor_data_length);
	memset((char *)sw_event.sensor_data,0,sw_event.sensor_data_length);

    /*Read switch status*/
    if ( sw_event.sensor_event == LED_ON )
       sprintf((char *)sw_event.sensor_data,"%s",SW_LED_ON);
    else
       sprintf((char *)sw_event.sensor_data,"%s",SW_LED_OFF);

    /*copy switch status to the buffer with delimiter "#"*/
    sprintf((char *)pSWEvent,"%s#%c#%d#%s#%d",(char *)sw_event.sensor_name,sw_event.sensor_type,
    		          sw_event.sensor_data_length,sw_event.sensor_data,sw_event.sensor_event);
    /*release buffer*/
	vPortFree(sw_event.sensor_data);

}

bsp_led_t get_toggle_channel() { return last_sw_toggle;}

