/***********************************************************************************************************************
* @file client.c
*
* @brief  Networking and Socket
*
* (C) Copyright 2021 SmartAxiom,Inc All Rights Reserved
*
*  SMARTAXIOM CONFIDENTIAL
*  The source code contained or described herein and all documents related to
*  the source code ("Material") are owned by SmartAxiom,Inc or its
*  suppliers or licensor's. Title to the Material remains with SmartAxiom, Inc
*  or its suppliers and licensor's. The Material contain trade secrets and proprietary
*  and confidential information of SmartAxiom or its suppliers and licensor's.
*  The Material is protected by worldwide copyright and trade secret laws and
*  treaty provisions. No part of the Material may be used, copied, reproduced, modified,
*  published, uploaded, posted,transmitted, distributed, or disclosed in any way without
*  SmartAxiom prior express written permission.
*
*  No license under any patent, copyright, trade secret or other intellectual
*  property right is granted to or conferred upon you by disclosure or delivery
*  of the Materials, either expressly, by implication, inducement, estoppel or
*  otherwise. Any license under such intellectual property rights must be
*  express and approved by SmartAxiom in writing.
*
*********************************************************************************************/

#include "bh_socket.h"
#include "beachhead.h"
#include "beachhead_config.h"

#define INADDR_ANY      0

/*----------------This is hold socket handle ---------*/
const TickType_t xSendTimeOut = 500/portTICK_PERIOD_MS;
const TickType_t xReceiveTimeout_ms = 500/portTICK_PERIOD_MS;
extern NetworkAddressingParameters_t xNd;

int socket_init(bh_socket_t *pSocket)
{
	int ret = SYNC_ERR_FAIL;

	ret = socket_open(pSocket);

	if ( ret == SYNC_OK) {
		APP_PRINT("\r\nConnect to Host at Port %d .......",PORT_B);
	} else  {
		APP_ERR_PRINT("\r\nUnable to connect host at Port %d ......",PORT_B);
	}
	return ret;
}

int socket_open(bh_socket_t *pSocket)
{

	BaseType_t ret;

	/*Initialize Port*/
    pSocket->xRemoteAddress.sin_port = FreeRTOS_htons(pSocket->vPort);

    /*set the broadcast address*/
    if (pSocket->bBroadcast) {
       pSocket->xRemoteAddress.sin_addr = xNd.ulBroadcastAddress;
    } else {
       pSocket->xRemoteAddress.sin_addr = FreeRTOS_inet_addr ((char *)pSocket->address);
    }

    /*Create a socket*/
    pSocket->xSocket = FreeRTOS_socket (FREERTOS_AF_INET,pSocket->socket_type, pSocket->socket_protocol);

    APP_PRINT("\r\n Heap size socket init: %d", xPortGetFreeHeapSize() );

    configASSERT( pSocket->xSocket != FREERTOS_INVALID_SOCKET);

    if (pSocket->xSocket != FREERTOS_INVALID_SOCKET) {

    	/*set the send time out*/
		ret = FreeRTOS_setsockopt ( pSocket->xSocket, 0,FREERTOS_SO_SNDTIMEO, &xSendTimeOut,0);
		/* Set the receive timeout. */
		ret = FreeRTOS_setsockopt ( pSocket->xSocket, 0, FREERTOS_SO_RCVTIMEO,&xReceiveTimeout_ms,0);

        if (pSocket->socket_protocol == FREERTOS_IPPROTO_UDP )	{
        	/*bind remote address and port*/
		    FreeRTOS_bind (pSocket->xSocket,(struct freertos_sockaddr *) &pSocket->xRemoteAddress,
		    		       sizeof(pSocket->xRemoteAddress));
        } else {
        	 FreeRTOS_connect (pSocket->xSocket,(struct freertos_sockaddr *) &pSocket->xRemoteAddress,
        	    		sizeof(pSocket->xRemoteAddress));
        }


    } else {
    	APP_ERR_PRINT("Unable to create socket %d", FREERTOS_INVALID_SOCKET);
    }

    return (!ret) ? SYNC_OK : SYNC_ERR_FAIL;

#ifdef FREERTOS_HTTP

#endif

}

/*---Connect to server---*/
int socket_connect(bh_socket_t *pSocket)
{

    BaseType_t ret;
    FreeRTOS_connect (pSocket->xSocket,(struct freertos_sockaddr *) &pSocket->xRemoteAddress,
    		sizeof(pSocket->xRemoteAddress));
    ret =  FreeRTOS_bind (pSocket->xSocket,(struct freertos_sockaddr *) &pSocket->xRemoteAddress,
    		sizeof(pSocket->xRemoteAddress));

    if (ret == pdTRUE ) {
      return SYNC_OK;
    }
    else {
      return SYNC_ERR_FAIL;
    }

}

/*----- shutdown socket connection -----*/
void socket_close(bh_socket_t *pSocket)
{

    if( pSocket->xSocket != FREERTOS_INVALID_SOCKET ) {
		FreeRTOS_shutdown( pSocket->xSocket, FREERTOS_SHUT_RDWR);
		vTaskDelay( (TickType_t) 250);
		FreeRTOS_closesocket (pSocket->xSocket);
		pSocket->xSocket = FREERTOS_INVALID_SOCKET;
    }

}

static int vStandardSend(bh_socket_t *pSocket,uint8_t *buff, BaseType_t buf_len)
{
    BaseType_t  xBytesSent = 0;

    if (pSocket->bBroadcast)
       pSocket->xRemoteAddress.sin_addr = xNd.ulBroadcastAddress;
    else
       pSocket->xRemoteAddress.sin_addr = FreeRTOS_inet_addr((char *)pSocket->address);

    pSocket->xRemoteAddress.sin_port = FreeRTOS_htons(pSocket->vPort);

    if (pSocket->socket_protocol == FREERTOS_IPPROTO_UDP ) {
   	   xBytesSent = (BaseType_t) FreeRTOS_sendto(pSocket->xSocket, buff, (size_t)buf_len,
   				                                    0,&pSocket->xRemoteAddress,
													sizeof(pSocket->xRemoteAddress));
   	} else {
   	   xBytesSent = (BaseType_t)FreeRTOS_send(pSocket->xSocket,buff,(size_t)buf_len,0 );
   	}

	return xBytesSent;
}

static int vStandardReceive(bh_socket_t *pSocket,uint8_t *buff, uint16_t recv_buf_len)
{
    int32_t iReturned = 0;
    size_t  remote_addr_len = sizeof(pSocket->xRemoteAddress);

    /* Receive into the buffer with ulFlags set to 0, so the FREERTOS_ZERO_COPY bit is clear. */
    if (pSocket->socket_protocol == FREERTOS_IPPROTO_UDP ) {
    	iReturned = FreeRTOS_recvfrom (pSocket->xSocket, buff,recv_buf_len,0, &pSocket->xRemoteAddress,
    	    		                         (socklen_t*)&remote_addr_len);
    } else {
        iReturned =  FreeRTOS_recv( pSocket->xSocket, buff,recv_buf_len , 0 );
    }

    return iReturned;
}

int socket_send(bh_socket_t *pSocket,uint8_t *buff,uint32_t buf_len)
{
   int ret = SYNC_ERR_FAIL;
   int xByteRet = 0;

   if (pSocket == NULL)
	   return ret;

   xByteRet = vStandardSend(pSocket,buff,(BaseType_t)buf_len);

   return (xByteRet == (int) buf_len) ? SYNC_OK: SYNC_ERR_FAIL;
}

int socket_receive(bh_socket_t *pSocket,uint8_t *recv_buff, uint16_t recv_buff_len)
{
	int ret = SYNC_ERR_FAIL;
	uint8_t bRet = 0;
	uint32_t retrieve = 0;

	if (recv_buff == NULL && pSocket == NULL)
		return ret;

	while (!bRet) {
	   ret = vStandardReceive(pSocket,recv_buff, recv_buff_len);

	   if (ret > 0 && ret <= recv_buff_len) {
			bRet = 1;
	   } else {
			retrieve++;
			if (retrieve == NUM_RETRIEVE) {
				bRet = 1;
				ret = SYNC_ERR_FAIL;
			}
		}
	}

	return ret;
}

/**
 *
 * @param pSocket
 * @param pAddress
 */

void GetRemoteAddress(bh_socket_t *pSocket,uint8_t *cIPAddressString)
{
#ifdef FREERTOS
	FreeRTOS_inet_ntoa(pSocket->xRemoteAddress.sin_addr, (char * )cIPAddressString);
#endif
}






