/***********************************************************************************************************************
* @file rtc.c
*
* @brief  RTC Hardware and driver interface source file
*
* (C) Copyright 2021 SmartAxiom,Inc All Rights Reserved
*
*  SMARTAXIOM CONFIDENTIAL
*  The source code contained or described herein and all documents related to
*  the source code ("Material") are owned by SmartAxiom,Inc or its
*  suppliers or licensor's. Title to the Material remains with SmartAxiom, Inc
*  or its suppliers and licensor's. The Material contain trade secrets and proprietary
*  and confidential information of SmartAxiom or its suppliers and licensor's.
*  The Material is protected by worldwide copyright and trade secret laws and
*  treaty provisions. No part of the Material may be used, copied, reproduced, modified,
*  published, uploaded, posted,transmitted, distributed, or disclosed in any way without
*  SmartAxiom prior express written permission.
*
*  No license under any patent, copyright, trade secret or other intellectual
*  property right is granted to or conferred upon you by disclosure or delivery
*  of the Materials, either expressly, by implication, inducement, estoppel or
*  otherwise. Any license under such intellectual property rights must be
*  express and approved by SmartAxiom in writing.
*
*/


#include "rtc.h"

static rtc_instance_ctrl_t g_rtc0_ctrl;

const rtc_error_adjustment_cfg_t g_rtc1_err_cfg =
{
		.adjustment_mode   = RTC_ERROR_ADJUSTMENT_MODE_AUTOMATIC,
		.adjustment_period = RTC_ERROR_ADJUSTMENT_PERIOD_10_SECOND,
		.adjustment_type   = RTC_ERROR_ADJUSTMENT_NONE,
		.adjustment_value  = 0,
};

const rtc_cfg_t g_rtc1_cfg =
{
		.clock_source            = RTC_CLOCK_SOURCE_LOCO,
		.freq_compare_value_loco = 255,
		.p_err_cfg               = &g_rtc1_err_cfg,
		.p_callback              = NULL,
		.p_context               = NULL,
		.alarm_ipl               = (BSP_IRQ_DISABLED),
		.periodic_ipl            = (BSP_IRQ_DISABLED),
		.carry_ipl               = (12),
#if defined(VECTOR_NUMBER_RTC_ALARM)
        .alarm_irq               = VECTOR_NUMBER_RTC_ALARM,
#else
		.alarm_irq               = FSP_INVALID_VECTOR,
#endif
#if defined(VECTOR_NUMBER_RTC_PERIOD)
        .periodic_irq            = VECTOR_NUMBER_RTC_PERIOD,
#else
		.periodic_irq            = FSP_INVALID_VECTOR,
#endif
#if defined(VECTOR_NUMBER_RTC_CARRY)
        .carry_irq               = VECTOR_NUMBER_RTC_CARRY,
#else
		.carry_irq               = FSP_INVALID_VECTOR,
#endif

};

/*Instance structure to use this module.*/
const rtc_instance_t g_rtc1 =
{
		.p_ctrl = &g_rtc0_ctrl,
		.p_cfg  = &g_rtc1_cfg,
		.p_api = &g_rtc_on_rtc
};

static rtc_time_t get_time;
const time_t start_val_time = 1605561768;

/**
 * @brief Initialize RTC controller
 * @return
 */
int rtc_user_init()
{
    int err = FSP_SUCCESS;
    rtc_time_t *l_time,tt;

    l_time = localtime_r(&start_val_time, &tt);

    /* Initialize the RTC module */
    err = R_RTC_Open(&g_rtc0_ctrl, &g_rtc1_cfg);

    if ( err == FSP_SUCCESS) {
    	/* Set the calendar time */
       R_RTC_CalendarTimeSet(&g_rtc0_ctrl,l_time);
       /* Get the calendar time */
       R_RTC_CalendarTimeGet(&g_rtc0_ctrl, &get_time);
    } else {
    	err = FSP_ERR_NOT_OPEN;
    }

    return err;
}


/**
 * @brief get current of the controller
 * */
time_t get_user_time(void)
{
    R_RTC_CalendarTimeGet(&g_rtc0_ctrl, &get_time);
    time_t count = mktime(&get_time);
    return count;
}

/**
 * @brief  set the current time of RTC controller
 * */
void set_user_time(time_t time)
{
    struct tm  *local_time,tt;
    local_time = localtime_r(&time,&tt);
    R_RTC_CalendarTimeSet(&g_rtc0_ctrl, local_time);
}




