/***********************************************************************************************************************
* @file  switches.c
*
* @brief Enable interrupts on pins connected to switches
*
* (C) Copyright 2021 SmartAxiom,Inc All Rights Reserved
*
*  SMARTAXIOM CONFIDENTIAL
*  The source code contained or described herein and all documents related to
*  the source code ("Material") are owned by SmartAxiom,Inc or its
*  suppliers or licensor's. Title to the Material remains with SmartAxiom, Inc
*  or its suppliers and licensor's. The Material contain trade secrets and proprietary
*  and confidential information of SmartAxiom or its suppliers and licensor's.
*  The Material is protected by worldwide copyright and trade secret laws and
*  treaty provisions. No part of the Material may be used, copied, reproduced, modified,
*  published, uploaded, posted,transmitted, distributed, or disclosed in any way without
*  SmartAxiom prior express written permission.
*
*  No license under any patent, copyright, trade secret or other intellectual
*  property right is granted to or conferred upon you by disclosure or delivery
*  of the Materials, either expressly, by implication, inducement, estoppel or
*  otherwise. Any license under such intellectual property rights must be
*  express and approved by SmartAxiom in writing.
*
*/

#include "switches.h"

fsp_err_t switches_init(void)
{
	fsp_err_t status = FSP_SUCCESS;

    status = R_ICU_ExternalIrqOpen(&pb_sw1_ctrl, &pb_sw1_cfg);
    if (FSP_SUCCESS != status)
    {
    	APP_ERR_PRINT("** R_ICU_ExternalIrqOpen pb_sw1 failed Error code = %d **\r\n", status);
        return status;
    }

    status = R_ICU_ExternalIrqEnable(&pb_sw1_ctrl);
    if (FSP_SUCCESS != status)
    {
    	APP_ERR_PRINT("** R_ICU_ExternalIrqEnable pb_sw1 failed Error code = %d **\r\n", status);
        return status;
    }

    status = R_ICU_ExternalIrqOpen(&pb_sw2_ctrl, &pb_sw2_cfg);
    if (FSP_SUCCESS != status)
    {
    	APP_ERR_PRINT("** R_ICU_ExternalIrqOpen pb_sw2 failed Error code = %d **\r\n", status);
        return status;
    }

    status = R_ICU_ExternalIrqEnable(&pb_sw2_ctrl);
    if (FSP_SUCCESS != status)
    {
    	APP_ERR_PRINT("** R_ICU_ExternalIrqEnable pb_sw2 failed Error code = %d **\r\n", status);
        return status;
    }
    return status;
}

void callback_switches(external_irq_callback_args_t *p_args)
{

	BaseType_t xHigherPriorityTaskWoken = pdFALSE;
    {
        if (PB_SW1_IRQ == p_args->channel) {
           xSemaphoreGiveFromISR(g_pb_sw1_semb, &xHigherPriorityTaskWoken);
        }
        else if (PB_SW2_IRQ == p_args->channel) {
           xSemaphoreGiveFromISR(g_pb_sw2_semb, &xHigherPriorityTaskWoken);
        }
    }
}
